

% DEVICE 2.1.0


% Class Device
classdef Device < handle
    
   % Public static Constants
   properties (Constant)
       
      % DHCP Status
      DHCP_DISABLED = 0;
      DHCP_ENABLED = 1;

   end
   

   % Public static Methods
   methods (Static)
       
      % Configure
      function configure(serial, varargin)
          
         % Check Parameters
         if nargin == 1
             
            % Set default Name, DHCP Status, Host IP Address, Subnet Mask, Gateway IP Address
            name = '';
            dhcp = Device.DHCP_ENABLED;
            ip_host = '192.168.1.1';
            ip_subnet_mask = '255.255.255.0';
            ip_gateway = '0.0.0.0';
             
         elseif nargin == 6

            % Set Name, DHCP Status, Host IP Address, Subnet Mask, Gateway IP Address
            name = varargin{1};
            dhcp = varargin{2};
            ip_host = varargin{3};
            ip_subnet_mask = varargin{4};
            ip_gateway = varargin{5};

         else

            % Error Handler
            this.error_(Device.ERROR_PARAMETER_);

         end
         
         % Configure
         code = calllib('device', 'configure', serial, name, strlength(name), dhcp, ...
            swapbytes(typecast(Device.ip_address_(ip_host).getAddress, 'uint32')), ...
            swapbytes(typecast(Device.ip_address_(ip_subnet_mask).getAddress, 'uint32')), ...
            swapbytes(typecast(Device.ip_address_(ip_gateway).getAddress, 'uint32')));

         % Error Handler
         Device.error_(code);          
          
      end

      
      % Get Driver
      function driver = driver()

         % String
         str = repmat(' ', 1, 1024);

         % Get Driver
         [code, manufacturer, manufacturer_length, version, version_length, release] = calllib('device', ...
            'get_driver', str, strlength(str), str, strlength(str), 0);

         % Error Handler
         Device.error_(code);

         % Return Driver
         driver = struct('manufacturer', manufacturer(1:manufacturer_length), 'version', version(1:version_length), ...
            'release', datestr(double(release + 1)));

      end


      % Find Devices in Subnet
      function device = find(serial)

         % Check Parameters
         if ~nargin

            % Serial Number List
            serial = [];

         end

         % Find Devices in Subnet
         [code, address, address_size] = calllib('device', 'find', zeros(1, 1024, 'uint32'), 1024, serial, ...
            numel(serial));

         % Error Handler
         Device.error_(code);

         % Initialize Device List
         device = [];

         % Parse Address List
         for i = 1:address_size

            % Insert into Device List
            device = [device Device(address(i))];

         end

      end


      % Initialize
      function init()

         % Get Architecture Info
         [~, max_array_size] = computer;

         % Check Operating System
         if ispc
         
            % Check for 32 or 64 Bit
            if (2 ^ 31) < max_array_size
                
               % Load 64-Bit Dynamic Link Library
               loadlibrary('lib/device64.dll', 'lib/device.h', 'alias', 'device');
               
            else
                
               % Load 32-Bit Dynamic Link Library
               loadlibrary('lib/device32.dll', 'lib/device.h', 'alias', 'device');

            end
            
         elseif ismac

            % Load 64-Bit shared Library
            loadlibrary('lib/device64.dylib', 'lib/device.h', 'alias', 'device');

         elseif isunix
         
            % Check for 32 or 64 Bit
            if (2 ^ 31) < max_array_size
                
               % Load 64-Bit shared Library
               loadlibrary('lib/device64.so', 'lib/device.h', 'alias', 'device');
               
            else
                
               % Load 32-Bit shared Library
               loadlibrary('lib/device32.so', 'lib/device.h', 'alias', 'device');

            end
         
         end

      end


      % Error Handler
      function error_(code)

         % Check Code
         if code

            % Identifier List
            identifier = {'Device:Error:Internal', 'Device:Error:Failed', 'Device:Error:Parameter', ...
               'Device:Error:Network', 'Device:Error:NotFound', 'Device:Error:Timeout', 'Device:Error:Aborted', ...
               'Device:Error:Reserved', 'Device:Error:Configuration'};

            % Message List
            message = {'An internal exception occured. Contact the manufacturer if the problem persists.', ...
               ['The operation could not be performed successfully. Check the device model, update the firmware, ' ...
               'reset the device, or contact the manufacturer if the problem persists.'], ...
               'One or more parameters are out of range or invalid. Check the validity of all parameters.', ...
               'The network communication failed. Check the network connectivity status and the network settings.', ...
               ['One or more devices could not be found. Check the network connectivity status and the network ' ...
               'settings.'], ...
               ['The operation could not be performed successfully since the maximum time has been reached. ' ...
               'Increasing the time-out value may solve the problem.'], ...
               'The operation was aborted by the user.', ...
               'The requested device is currently reserved. Try again later.', ...
               'The device is not properly configured. Check the parameters and consult the manual.'};

            % Check Code
            if numel(message) < code

               % Internal Error
               code = Device.ERROR_INTERNAL_;

            end

            % Exception
            throw(MException(identifier{code}, message{code}));

         end

      end

   end


   % Public Methods
   methods

      % Constructor
      function this = Device(address)

         % Check Parameter
         if isa(address, 'Device')

            % Get Address from Device
            address = address.address;

         end

         % Initialize
         this.address(address);
         this.timeout_ = 1000;

      end


      % Get / Set Address
      function address_ = address(this, address)

         % Check Parameters
         if nargin == 1

            % Return Address
            address_ = Device.ip_address_(this.address_);

         else

            % Set Address
            this.address_ = swapbytes(typecast(Device.ip_address_(address).getAddress, 'uint32'));

         end

      end

      
      % Burn Tag
      function burn(this, tag)

         % Reset Device
         code = calllib('device', 'burn', this.address_, tag, strlength(tag));

         % Error Handler
         this.error_(code);

      end

      
      % Get Information
      function info = info(this)

         % String
         str = repmat(' ', 1, 1024);

         % Get Information
         [code, model, model_length, description, description_length, manufacturer, manufacturer_length, serial, ...
            revision, revision_length, cpu, cpu_length, frequency, memory, network, network_length, version, ...
            version_length, release, calibration, expiration, uptime_reset, uptime_total, cycles, eth_host, ...
            ip_host, ip_subnet_mask, ip_gateway, dhcp, hostname, hostname_length, domain, domain_length, name, ...
            name_length, tag, tag_length] = calllib('device', 'get_info', this.address_, str, strlength(str), str, ...
            strlength(str), str, strlength(str), 0, str, strlength(str), str, strlength(str), 0, 0, str, ...
            strlength(str), str, strlength(str), 0, 0, 0, 0, 0, 0, zeros(1, 6, 'uint8'), 0, 0, 0, 0, str, ...
            strlength(str), str, strlength(str), str, strlength(str), str, strlength(str));

         % Error Handler
         this.error_(code);

         % Check Calibration Date
         if calibration

            % Format Calibration Date String
            calibration = datestr(double(calibration + 1));

         else

            % Undefined Calibration Date
            calibration = 'undefined';

         end

         % Check Expiration of Calibration Date
         if expiration

            % Format Expiration of Calibration Date String
            expiration = datestr(double(expiration + 1));

         else

            % Undefined Expiration of Calibration Date
            expiration = 'undefined';

         end

         % Return Information
         info = struct('model', model(1:model_length), 'description', description(1:description_length), ...
            'manufacturer', manufacturer(1:manufacturer_length), 'serial', dec2hex(serial, 8), 'revision', ...
            revision(1:revision_length), 'cpu', cpu(1:cpu_length), 'frequency', frequency, 'memory', memory, ...
            'network', network(1:network_length), 'version', version(1:version_length), 'release', ...
            datestr(double(release) + 1.0), 'calibration', calibration, 'expiration', expiration, 'uptime_reset', ...
            uptime_reset, 'uptime_total', uptime_total, 'cycles', cycles, 'eth_host', ...
            getfield(num2str(eth_host, '%02X:'), {1:17}), 'ip_host', Device.ip_address_(ip_host), 'ip_subnet_mask', ...
            Device.ip_address_(ip_subnet_mask), 'ip_gateway', Device.ip_address_(ip_gateway), 'dhcp', dhcp, ...
            'hostname', hostname(1:hostname_length), 'domain', domain(1:domain_length), 'name', ...
            name(1:name_length), 'tag', tag(1:tag_length));

      end


      % Reset Device
      function reset(this)

         % Reset Device
         code = calllib('device', 'reset', this.address_);

         % Error Handler
         this.error_(code);

      end


      % Get Status
      function [property, ok] = status(this)

         % String, Numeric Array
         str = repmat(' ', 1, 1024);
         num = zeros(1, 255, 'single');
         
         % Get Status
         [code, symbol, symbol_length, description, description_length, unit, unit_length, typical, min, max, value, ...
            size, ok] = calllib('device', 'get_status', this.address_, str, strlength(str), str, strlength(str), ...
            str, strlength(str), num, num, num, num, 255, 0);

         % Error Handler
         this.error_(code);

         % Split Strings
         symbol = textscan(symbol(1:symbol_length), '%s', 'Delimiter', '\t');
         description = textscan(description(1:description_length), '%s', 'Delimiter', '\t');
         unit = textscan(unit(1:unit_length), '%s', 'Delimiter', '\t');

         % Property List
         property = [];

         % Parse Property List
         for i = 1:size

            % Update Property List
            property = [property struct('symbol', symbol{1}(i), 'description', description{1}(i), 'unit', ...
               unit{1}(i), 'typical', typical(i), 'min', min(i), 'max', max(i), 'value', value(i))];

         end

      end


      % Get / Set Timeout
      function timeout_ = timeout(this, timeout)

         % Check Parameters
         if nargin == 1

            % Get Timeout
            [code, timeout_] = calllib('device', 'get_timeout', this.address_, 0);

         else

            % Set Timeout
            code = calllib('device', 'set_timeout', this.address_, timeout);

         end

         % Error Handler
         this.error_(code);

      end

   end
   
   
   % Protected Properties
   properties (Access = protected)

      % Variables
      address_;

   end   


   % Private static Constants
   properties (Constant, Access = private)
       
      % Error Codes
      ERROR_OK_ = 0;
      ERROR_INTERNAL_ = 1;
      ERROR_FAILED_ = 2;
      ERROR_PARAMETER_ = 3;
      ERROR_NETWORK_ = 4;
      ERROR_NOT_FOUND_ = 5;
      ERROR_TIMEOUT_ = 6;
      ERROR_ABORTED_ = 7;
      ERROR_RESERVED_ = 8;
      ERROR_CONFIGURATION_ = 9;

   end
   
   
   % Private static Methods
   methods (Static, Access = private)
       
      % Convert to IP Address
      function ip_address_ = ip_address_(ip_address)

         % Check Address
         if isnumeric(ip_address)

            % Return Address
            ip_address_ = java.net.InetAddress.getByAddress(typecast(swapbytes(uint32(ip_address)), 'uint8'));

        elseif isa(ip_address, 'java.net.Inet4Address')

           % Return Address
           ip_address_ = ip_address;

        else

           % Return Address
           ip_address_ = java.net.InetAddress.getByName(ip_address);

        end

      end       
       
   end
   

   % Private Properties
   properties (Access = private)

      % Variables
      timeout_;

   end   

end
