

% TMP02 SENSOR - QUAD RTD MONITOR 1.1.0


% Class TMP02_Sensor
classdef TMP02_Sensor < handle

   % Public static Constants
   properties (Constant)

      % Excitation Currents [A]
      CURRENT_DEFAULT = NaN;
      CURRENT_NONE = 0.0;
      CURRENT_10UA = 0.00001;
      CURRENT_1MA = 0.001;

      % Nominal Resistance Limits [Ohms]
      RESISTANCE_DEFAULT = 100.0;
      RESISTANCE_MIN = 1.0;
      RESISTANCE_MAX = 1000.0;

      % Types
      TYPE_NONE = 0;
      TYPE_CUSTOM = 1;
      TYPE_PT = 2;
      TYPE_DT470 = 3;
      TYPE_DT670 = 4;
      TYPE_KTY81_1 = 5;
      TYPE_KTY81_2 = 6;
      TYPE_KTY82_1 = 7;
      TYPE_KTY82_2 = 8;
      TYPE_KTY83_1 = 9;
      TYPE_KTY84_1 = 10;

      % Voltage Ranges [V]
      VOLTAGE_DEFAULT = NaN;
      VOLTAGE_0V5 = 0.5;
      VOLTAGE_1V = 1.0;
      VOLTAGE_2V5 = 2.5;
      VOLTAGE_5V = 5.0;

   end


   % Public Member Methods
   methods

      % Constructor
      function this = TMP02_Sensor(varargin)
      
         % Initialize
         this.type(this.TYPE_NONE);
         this.voltage(this.VOLTAGE_DEFAULT);
         this.current(this.CURRENT_DEFAULT);
         this.resistance(this.RESISTANCE_DEFAULT);

         % Check Parameters
         if nargin == 1

            % Initialize
            this.type(varargin{1});

         elseif nargin == 2

            % Initialize
            this.type(varargin{1});
            this.resistance(varargin{2});

         elseif nargin == 3

            % Initialize
            this.type(varargin{1});
            this.voltage(varargin{2});
            this.current(varargin{3});

         elseif nargin

            % Error Handler
            Device.error_(Device.ERROR_PARAMETER_);

         end

      end


      % Get / Set Excitation Current
      function current_ = current(this, current)

         % Check Parameters
         if nargin == 1

            % Return Excitation Current
            current_ = this.current_;

         else

            % Check Excitation Current
            if ~isnan(current) && (current ~= this.CURRENT_NONE) && (current ~= this.CURRENT_10UA) && ...
               (current ~= this.CURRENT_1MA)

               % Error Handler
               Device.error_(Device.ERROR_PARAMETER_);

            end

            % Set Excitation Current
            this.current_ = current;

         end

      end


      % Get / Set nominal Resistance
      function resistance_ = resistance(this, resistance)

         % Check Parameters
         if nargin == 1

            % Return nominal Resistance
            resistance_ = this.resistance_;

         else

            % Check nominal Resistance
            if (resistance < this.RESISTANCE_MIN) || (this.RESISTANCE_MAX < resistance)

               % Error Handler
               Device.error_(Device.ERROR_PARAMETER_);

            end

            % Set nominal Resistance
            this.resistance_ = resistance;

         end

      end


      % Get / Set Type
      function type_ = type(this, type)

         % Check Parameters
         if nargin == 1

            % Return Type
            type_ = this.type_;

         else

            % Check Type
            if (type ~= this.TYPE_NONE) && (type ~= this.TYPE_CUSTOM) && (type ~= this.TYPE_PT) && ...
               (type ~= this.TYPE_DT470) && (type ~= this.TYPE_DT670) && (type ~= this.TYPE_KTY81_1) && ...
               (type ~= this.TYPE_KTY81_2) && (type ~= this.TYPE_KTY82_1) && (type ~= this.TYPE_KTY82_2) && ...
               (type ~= this.TYPE_KTY83_1) && (type ~= this.TYPE_KTY84_1)

               % Error Handler
               Device.error_(Device.ERROR_PARAMETER_);

            end

            % Set Type
            this.type_ = type;

         end

      end


      % Get / Set Voltage Range
      function voltage_ = voltage(this, voltage)

         % Check Parameters
         if nargin == 1

            % Return Voltage Range
            voltage_ = this.voltage_;

         else

            % Check Voltage Range
            if ~isnan(voltage) && (voltage ~= this.VOLTAGE_0V5) && (voltage ~= this.VOLTAGE_1V) && ...
               (voltage ~= this.VOLTAGE_2V5) && (voltage ~= this.VOLTAGE_5V)

               % Error Handler
               Device.error_(Device.ERROR_PARAMETER_);

            end

            % Set Voltage Range
            this.voltage_ = voltage;

         end

      end

   end


   % Private Properties
   properties (Access = private)

      % Variables
      current_;
      resistance_;
      type_;
      voltage_;

   end

end
